// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

import { getArray } from './bnInput';
import EcKeyBase from './ecKeyBase';

const publicHeader = "MFYwEAYHKoZIzj0CAQYFK4EEAAoDQgAE";
const privateHeader = "MHQCAQEEIA==";

const public9256 = 'MFkwEwYHKoZIzj0CAQYIKoZIzj0DAQcDQgAEAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA==';

export default class EcKeyFormat {
    /**
     * @param { EcKeyBase<T, U> } key
     * @template { import('./ecPointBase').default } T
     * @template {EcKeyBase<T,U>} U
     */
    static PemPublic(key) {
        if (key.curve.name === 'secp256k1') { 
            return Buffer.concat([Buffer.from(publicHeader, 'base64'), Buffer.from(key.y.toArray())]).toString("base64");
        }
        else if (key.curve.name === 'p256') {
            var buffer = Buffer.from(public9256, 'base64');
            buffer.set(key.y.toArray(), 27);
            return buffer.toString('base64');
        }
        
        throw new Error(`Key type ${key.curve.name} is not supported for serialization`);
    }

    /**
     * @param { EcKeyBase<T, U> } key
     * @template { import('./ecPointBase').default } T
     * @template {EcKeyBase<T,U>} U
     */
    static PemPrivate(key) {
        if (key.curve.name === 'secp256k1')
            return Buffer.concat([Buffer.from(privateHeader, 'base64')
                , Buffer.from(getArray(key.x))
                , Buffer.from("oAcGBSuBBAAKoUQDQgAE", 'base64')
                , Buffer.from(key.y.toArray())]).toString("base64");

        throw new Error(`Key type ${key.curve.name} is not supported for serialization`);
    }
}